#ifndef QT_FEATURES_Test_qtbase_src_testlib_qttestlib_config_h_H
#define QT_FEATURES_Test_qtbase_src_testlib_qttestlib_config_h_H

#define QT_FEATURE_testlib_selfcover -1

#define QT_FEATURE_itemmodeltester 1

#define QT_FEATURE_valgrind 1

#define QT_FEATURE_batch_test_support -1


#endif // QT_FEATURES_Test_qtbase_src_testlib_qttestlib_config_h_H
