// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTMULTIMEDIAGLOBAL_H
#define QTMULTIMEDIAGLOBAL_H

#include <QtGui/qtguiglobal.h>
#include <QtMultimedia/qtmultimedia-config.h>
#include <QtMultimedia/qtmultimediaexports.h>

#endif // QTMULTIMEDIAGLOBAL_H
