// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
// Qt-Security score:significant reason:default

#ifndef QTQUICKTIMELINEBLENDTREESGLOBAL_P_H
#define QTQUICKTIMELINEBLENDTREESGLOBAL_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include "qtquicktimelineblendtreesglobal.h"
#include <QtQuickTimelineBlendTrees/private/qtquicktimelineblendtreesexports_p.h>

#endif // QTQUICKTIMELINEBLENDTREESGLOBAL_P_H
