// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
// Qt-Security score:significant reason:default

#ifndef QTQUICKTIMELINEBLENDTREESGLOBAL_H
#define QTQUICKTIMELINEBLENDTREESGLOBAL_H

#include <QtCore/qglobal.h>
#include <QtQuickTimelineBlendTrees/qtquicktimelineblendtreesexports.h>

#endif // QTQUICKTIMELINEBLENDTREESGLOBAL_H
