// This file was generated by qtwaylandscanner
// source file is xdg-system-bell-v1.xml

#ifndef QT_WAYLAND_XDG_SYSTEM_BELL_V1
#define QT_WAYLAND_XDG_SYSTEM_BELL_V1

#include <QtWaylandClient/private/wayland-xdg-system-bell-v1-client-protocol.h>
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")
#if !defined(Q_WAYLAND_CLIENT_XDG_SYSTEM_BELL_V1_EXPORT)
#  if defined(QT_SHARED) && !defined(QT_STATIC)
#    if defined(QT_BUILD_WAYLANDCLIENT_LIB)
#      define Q_WAYLAND_CLIENT_XDG_SYSTEM_BELL_V1_EXPORT Q_DECL_EXPORT
#    else
#      define Q_WAYLAND_CLIENT_XDG_SYSTEM_BELL_V1_EXPORT Q_DECL_IMPORT
#    endif
#  else
#    define Q_WAYLAND_CLIENT_XDG_SYSTEM_BELL_V1_EXPORT
#  endif
#endif

namespace QtWayland {
    class Q_WAYLAND_CLIENT_XDG_SYSTEM_BELL_V1_EXPORT xdg_system_bell_v1
    {
    public:
        xdg_system_bell_v1(struct ::wl_registry *registry, uint32_t id, int version);
        xdg_system_bell_v1(struct ::xdg_system_bell_v1 *object);
        xdg_system_bell_v1();

        virtual ~xdg_system_bell_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::xdg_system_bell_v1 *object);

        struct ::xdg_system_bell_v1 *object() { return m_xdg_system_bell_v1; }
        const struct ::xdg_system_bell_v1 *object() const { return m_xdg_system_bell_v1; }
        static xdg_system_bell_v1 *fromObject(struct ::xdg_system_bell_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void destroy();
        void ring(struct ::wl_surface *surface);

    private:
        struct ::xdg_system_bell_v1 *m_xdg_system_bell_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
