// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTESTSYSTEM_H
#define QTESTSYSTEM_H

#include <QtTest/qtestcase.h>

#include <QtCore/qtestsupport_core.h>
#ifdef QT_GUI_LIB
#  include <QtGui/qtestsupport_gui.h>
#endif
#ifdef QT_WIDGETS_LIB
#  include <QtWidgets/qtestsupport_widgets.h>
#endif

QT_BEGIN_NAMESPACE
QT_END_NAMESPACE

#endif

