// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:significant

// this file was generated by the generate.ts script

#ifndef QLSPNOTIFYSIGNALS_P_H
#define QLSPNOTIFYSIGNALS_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtLanguageServer/private/qlanguageserverprotocol_p.h>

QT_BEGIN_NAMESPACE

class Q_LANGUAGESERVER_EXPORT QLspNotifySignals : public QObject
{
    Q_OBJECT
public:
    QLspNotifySignals(QObject *parent = nullptr) : QObject(parent) { }
    void registerHandlers(QLanguageServerProtocol *protocol);
signals:
    void receivedCancelNotification(const QLspSpecification::Notifications::CancelParamsType &);
    void receivedProgressNotification(const QLspSpecification::Notifications::ProgressParamsType &);
    void receivedInitializedNotification(
            const QLspSpecification::Notifications::InitializedParamsType &);
    void receivedExitNotification(const QLspSpecification::Notifications::ExitParamsType &);
    void receivedLogTraceNotification(const QLspSpecification::Notifications::LogTraceParamsType &);
    void receivedSetTraceNotification(const QLspSpecification::Notifications::SetTraceParamsType &);
    void receivedShowMessageNotification(
            const QLspSpecification::Notifications::ShowMessageParamsType &);
    void
    receivedLogMessageNotification(const QLspSpecification::Notifications::LogMessageParamsType &);
    void receivedWorkDoneProgressCancelNotification(
            const QLspSpecification::Notifications::WorkDoneProgressCancelParamsType &);
    void receivedTelemetryEventNotification(
            const QLspSpecification::Notifications::TelemetryEventParamsType &);
    void receivedDidChangeWorkspaceFoldersNotification(
            const QLspSpecification::Notifications::DidChangeWorkspaceFoldersParamsType &);
    void receivedDidChangeConfigurationNotification(
            const QLspSpecification::Notifications::DidChangeConfigurationParamsType &);
    void receivedDidChangeWatchedFilesNotification(
            const QLspSpecification::Notifications::DidChangeWatchedFilesParamsType &);
    void receivedCreateFilesNotification(
            const QLspSpecification::Notifications::CreateFilesParamsType &);
    void receivedRenameFilesNotification(
            const QLspSpecification::Notifications::RenameFilesParamsType &);
    void receivedDeleteFilesNotification(
            const QLspSpecification::Notifications::DeleteFilesParamsType &);
    void receivedDidOpenTextDocumentNotification(
            const QLspSpecification::Notifications::DidOpenTextDocumentParamsType &);
    void receivedDidChangeTextDocumentNotification(
            const QLspSpecification::Notifications::DidChangeTextDocumentParamsType &);
    void receivedWillSaveTextDocumentNotification(
            const QLspSpecification::Notifications::WillSaveTextDocumentParamsType &);
    void receivedDidSaveTextDocumentNotification(
            const QLspSpecification::Notifications::DidSaveTextDocumentParamsType &);
    void receivedDidCloseTextDocumentNotification(
            const QLspSpecification::Notifications::DidCloseTextDocumentParamsType &);
    void receivedPublishDiagnosticsNotification(
            const QLspSpecification::Notifications::PublishDiagnosticsParamsType &);
};

QT_END_NAMESPACE

#endif // QLSPNOTIFYSIGNALS_P_H
