// Copyright (C) 2019 The Qt Company Ltd.
// Copyright (C) 2018 Intel Corporation.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:significant reason:header-decls-only

#ifndef QCBORSTREAM_H
#define QCBORSTREAM_H

#include <QtCore/qglobal.h>

#if QT_CONFIG(cborstreamreader)
#include <QtCore/qcborstreamreader.h>
#endif

#if QT_CONFIG(cborstreamwriter)
#include <QtCore/qcborstreamwriter.h>
#endif

QT_BEGIN_NAMESPACE
QT_END_NAMESPACE

#endif // QCBORSTREAM_H
