// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QTQUICKGLOBAL_H
#define QTQUICKGLOBAL_H

#include <QtQml/qtqmlglobal.h>
#include <QtGui/qtguiglobal.h>
#include <QtQuick/qtquick-config.h>
#include <QtQuick/qtquickexports.h>

#endif // QTQUICKGLOBAL_H
