#ifndef QT_FEATURES_Sql_qtbase_src_sql_qtsql_config_h_H
#define QT_FEATURES_Sql_qtbase_src_sql_qtsql_config_h_H

#define QT_FEATURE_sqlmodel 1


#endif // QT_FEATURES_Sql_qtbase_src_sql_qtsql_config_h_H
