#ifndef QT_FEATURES_WaylandCompositor_qtwayland_src_compositor_qtwaylandcompositor_config_h_H
#define QT_FEATURES_WaylandCompositor_qtwayland_src_compositor_qtwaylandcompositor_config_h_H

#define QT_FEATURE_wayland_compositor_quick 1


#endif // QT_FEATURES_WaylandCompositor_qtwayland_src_compositor_qtwaylandcompositor_config_h_H
