/* This file was generated by syncqt. */
#ifndef QT_QTQUICKTIMELINEBLENDTREES_VERSION_H
#define QT_QTQUICKTIMELINEBLENDTREES_VERSION_H

#define QTQUICKTIMELINEBLENDTREES_VERSION_STR "6.11.0"

#define QTQUICKTIMELINEBLENDTREES_VERSION 0x0601100

#endif // QT_QTQUICKTIMELINEBLENDTREES_VERSION_H
