// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QVARIANTLIST_H
#define QVARIANTLIST_H

#if 0
#pragma qt_class(QVariantList)
#endif

#include <QtCore/qglobal.h>

#include <QtCore/QList>
#include <QtCore/QVariant>

QT_BEGIN_NAMESPACE
QT_END_NAMESPACE

#endif
