#ifndef QT_FEATURES_PrintSupport_qtbase_src_printsupport_qtprintsupport_config_h_H
#define QT_FEATURES_PrintSupport_qtbase_src_printsupport_qtprintsupport_config_h_H

#define QT_FEATURE_cups -1

#define QT_FEATURE_cupsjobwidget -1

#define QT_FEATURE_printer 1

#define QT_FEATURE_printpreviewwidget 1

#define QT_FEATURE_printdialog 1

#define QT_FEATURE_printpreviewdialog 1

#define QT_NO_CUPS 1
#define QT_NO_CUPSJOBWIDGET 1

#endif // QT_FEATURES_PrintSupport_qtbase_src_printsupport_qtprintsupport_config_h_H
