#ifndef QT_FEATURES_QuickTemplates2_qtdeclarative_src_quicktemplates_qtquicktemplates2_config_p_h_H
#define QT_FEATURES_QuickTemplates2_qtdeclarative_src_quicktemplates_qtquicktemplates2_config_p_h_H

#define QT_FEATURE_quicktemplates2_hover 1

#define QT_FEATURE_quicktemplates2_multitouch 1

#define QT_FEATURE_quicktemplates2_calendar 1

#define QT_FEATURE_quicktemplates2_container 1


#endif // QT_FEATURES_QuickTemplates2_qtdeclarative_src_quicktemplates_qtquicktemplates2_config_p_h_H
