// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
// Qt-Security score:significant reason:default


#ifndef QTQUICK3DXRGLOBAL_H
#define QTQUICK3DXRGLOBAL_H

#include <QtGui/qtguiglobal.h>

#include <QtQuick3DXr/qtquick3dxrexports.h>

#endif // QTQUICK3DXRGLOBAL_H
