#ifndef QT_FEATURES_WaylandGlobalPrivate_qtbase_src_plugins_platforms_wayland_qtwaylandglobal_config_h_H
#define QT_FEATURES_WaylandGlobalPrivate_qtbase_src_plugins_platforms_wayland_qtwaylandglobal_config_h_H


#endif // QT_FEATURES_WaylandGlobalPrivate_qtbase_src_plugins_platforms_wayland_qtwaylandglobal_config_h_H
