// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
// Qt-Security score:significant reason:default


#ifndef QTQUICK3DPARTICLESGLOBAL_P_H
#define QTQUICK3DPARTICLESGLOBAL_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists purely as an
// implementation detail.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <QtQuick3DParticles/qtquick3dparticlesglobal.h>
#include <QtQuick3DParticles/qtquick3dparticlesexports.h>

QT_BEGIN_NAMESPACE

void Q_QUICK3DPARTICLES_EXPORT qml_register_types_QtQuick3D_Particles3D();

QT_END_NAMESPACE

#endif // QTQUICK3DPARTICLESGLOBAL_P_H
