// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#ifndef QFUNCTIONPOINTER_H
#define QFUNCTIONPOINTER_H

#include <QtCore/qtconfigmacros.h>

#if 0
#pragma qt_class(QFunctionPointer)
#endif

#if defined(__cplusplus)

QT_BEGIN_NAMESPACE

typedef void (*QFunctionPointer)();

QT_END_NAMESPACE

#endif // __cplusplus

#endif // QFUNCTIONPOINTER_H
