#ifndef QT_FEATURES_Sensors_qtsensors_src_sensors_qtsensors_config_h_H
#define QT_FEATURES_Sensors_qtsensors_src_sensors_qtsensors_config_h_H


#endif // QT_FEATURES_Sensors_qtsensors_src_sensors_qtsensors_config_h_H
