/* This file was generated by syncqt. */
#ifndef QT_QTMULTIMEDIATESTLIB_VERSION_H
#define QT_QTMULTIMEDIATESTLIB_VERSION_H

#define QTMULTIMEDIATESTLIB_VERSION_STR "6.11.0"

#define QTMULTIMEDIATESTLIB_VERSION 0x0601100

#endif // QT_QTMULTIMEDIATESTLIB_VERSION_H
