// This file was generated by qtwaylandscanner
// source file is color-management-v1.xml

#ifndef QT_WAYLAND_COLOR_MANAGEMENT_V1
#define QT_WAYLAND_COLOR_MANAGEMENT_V1

#include <QtWaylandClient/private/wayland-color-management-v1-client-protocol.h>
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")
#if !defined(Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT)
#  if defined(QT_SHARED) && !defined(QT_STATIC)
#    if defined(QT_BUILD_WAYLANDCLIENT_LIB)
#      define Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT Q_DECL_EXPORT
#    else
#      define Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT Q_DECL_IMPORT
#    endif
#  else
#    define Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT
#  endif
#endif

namespace QtWayland {
    class Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT wp_color_manager_v1
    {
    public:
        wp_color_manager_v1(struct ::wl_registry *registry, uint32_t id, int version);
        wp_color_manager_v1(struct ::wp_color_manager_v1 *object);
        wp_color_manager_v1();

        virtual ~wp_color_manager_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::wp_color_manager_v1 *object);

        struct ::wp_color_manager_v1 *object() { return m_wp_color_manager_v1; }
        const struct ::wp_color_manager_v1 *object() const { return m_wp_color_manager_v1; }
        static wp_color_manager_v1 *fromObject(struct ::wp_color_manager_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_unsupported_feature = 0, // request not supported
            error_surface_exists = 1, // color management surface exists already
        };

        enum render_intent {
            render_intent_perceptual = 0, // perceptual
            render_intent_relative = 1, // media-relative colorimetric
            render_intent_saturation = 2, // saturation
            render_intent_absolute = 3, // ICC-absolute colorimetric
            render_intent_relative_bpc = 4, // media-relative colorimetric + black point compensation
        };

        enum feature {
            feature_icc_v2_v4 = 0, // create_icc_creator request
            feature_parametric = 1, // create_parametric_creator request
            feature_set_primaries = 2, // parametric set_primaries request
            feature_set_tf_power = 3, // parametric set_tf_power request
            feature_set_luminances = 4, // parametric set_luminances request
            feature_set_mastering_display_primaries = 5,
            feature_extended_target_volume = 6,
            feature_windows_scrgb = 7, // create_windows_scrgb request
        };

        enum primaries {
            primaries_srgb = 1,
            primaries_pal_m = 2,
            primaries_pal = 3,
            primaries_ntsc = 4,
            primaries_generic_film = 5,
            primaries_bt2020 = 6,
            primaries_cie1931_xyz = 7,
            primaries_dci_p3 = 8,
            primaries_display_p3 = 9,
            primaries_adobe_rgb = 10,
        };

        enum transfer_function {
            transfer_function_bt1886 = 1,
            transfer_function_gamma22 = 2,
            transfer_function_gamma28 = 3,
            transfer_function_st240 = 4,
            transfer_function_ext_linear = 5,
            transfer_function_log_100 = 6,
            transfer_function_log_316 = 7,
            transfer_function_xvycc = 8,
            transfer_function_srgb = 9,
            transfer_function_ext_srgb = 10,
            transfer_function_st2084_pq = 11,
            transfer_function_st428 = 12,
            transfer_function_hlg = 13,
        };

        void destroy();
        struct ::wp_color_management_output_v1 *get_output(struct ::wl_output *output);
        struct ::wp_color_management_surface_v1 *get_surface(struct ::wl_surface *surface);
        struct ::wp_color_management_surface_feedback_v1 *get_surface_feedback(struct ::wl_surface *surface);
        struct ::wp_image_description_creator_icc_v1 *create_icc_creator();
        struct ::wp_image_description_creator_params_v1 *create_parametric_creator();
        struct ::wp_image_description_v1 *create_windows_scrgb();

    protected:
        virtual void wp_color_manager_v1_supported_intent(uint32_t render_intent);
        virtual void wp_color_manager_v1_supported_feature(uint32_t feature);
        virtual void wp_color_manager_v1_supported_tf_named(uint32_t tf);
        virtual void wp_color_manager_v1_supported_primaries_named(uint32_t primaries);
        virtual void wp_color_manager_v1_done();

    private:
        void init_listener();
        static const struct wp_color_manager_v1_listener m_wp_color_manager_v1_listener;
        static void handle_supported_intent(
            void *data,
            struct ::wp_color_manager_v1 *object,
            uint32_t render_intent);
        static void handle_supported_feature(
            void *data,
            struct ::wp_color_manager_v1 *object,
            uint32_t feature);
        static void handle_supported_tf_named(
            void *data,
            struct ::wp_color_manager_v1 *object,
            uint32_t tf);
        static void handle_supported_primaries_named(
            void *data,
            struct ::wp_color_manager_v1 *object,
            uint32_t primaries);
        static void handle_done(
            void *data,
            struct ::wp_color_manager_v1 *object);
        struct ::wp_color_manager_v1 *m_wp_color_manager_v1;
    };

    class Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT wp_color_management_output_v1
    {
    public:
        wp_color_management_output_v1(struct ::wl_registry *registry, uint32_t id, int version);
        wp_color_management_output_v1(struct ::wp_color_management_output_v1 *object);
        wp_color_management_output_v1();

        virtual ~wp_color_management_output_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::wp_color_management_output_v1 *object);

        struct ::wp_color_management_output_v1 *object() { return m_wp_color_management_output_v1; }
        const struct ::wp_color_management_output_v1 *object() const { return m_wp_color_management_output_v1; }
        static wp_color_management_output_v1 *fromObject(struct ::wp_color_management_output_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void destroy();
        struct ::wp_image_description_v1 *get_image_description();

    protected:
        virtual void wp_color_management_output_v1_image_description_changed();

    private:
        void init_listener();
        static const struct wp_color_management_output_v1_listener m_wp_color_management_output_v1_listener;
        static void handle_image_description_changed(
            void *data,
            struct ::wp_color_management_output_v1 *object);
        struct ::wp_color_management_output_v1 *m_wp_color_management_output_v1;
    };

    class Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT wp_color_management_surface_v1
    {
    public:
        wp_color_management_surface_v1(struct ::wl_registry *registry, uint32_t id, int version);
        wp_color_management_surface_v1(struct ::wp_color_management_surface_v1 *object);
        wp_color_management_surface_v1();

        virtual ~wp_color_management_surface_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::wp_color_management_surface_v1 *object);

        struct ::wp_color_management_surface_v1 *object() { return m_wp_color_management_surface_v1; }
        const struct ::wp_color_management_surface_v1 *object() const { return m_wp_color_management_surface_v1; }
        static wp_color_management_surface_v1 *fromObject(struct ::wp_color_management_surface_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_render_intent = 0, // unsupported rendering intent
            error_image_description = 1, // invalid image description
            error_inert = 2, // forbidden request on inert object
        };

        void destroy();
        void set_image_description(struct ::wp_image_description_v1 *image_description, uint32_t render_intent);
        void unset_image_description();

    private:
        struct ::wp_color_management_surface_v1 *m_wp_color_management_surface_v1;
    };

    class Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT wp_color_management_surface_feedback_v1
    {
    public:
        wp_color_management_surface_feedback_v1(struct ::wl_registry *registry, uint32_t id, int version);
        wp_color_management_surface_feedback_v1(struct ::wp_color_management_surface_feedback_v1 *object);
        wp_color_management_surface_feedback_v1();

        virtual ~wp_color_management_surface_feedback_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::wp_color_management_surface_feedback_v1 *object);

        struct ::wp_color_management_surface_feedback_v1 *object() { return m_wp_color_management_surface_feedback_v1; }
        const struct ::wp_color_management_surface_feedback_v1 *object() const { return m_wp_color_management_surface_feedback_v1; }
        static wp_color_management_surface_feedback_v1 *fromObject(struct ::wp_color_management_surface_feedback_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_inert = 0, // forbidden request on inert object
            error_unsupported_feature = 1, // attempted to use an unsupported feature
        };

        void destroy();
        struct ::wp_image_description_v1 *get_preferred();
        struct ::wp_image_description_v1 *get_preferred_parametric();

    protected:
        virtual void wp_color_management_surface_feedback_v1_preferred_changed(uint32_t identity);

    private:
        void init_listener();
        static const struct wp_color_management_surface_feedback_v1_listener m_wp_color_management_surface_feedback_v1_listener;
        static void handle_preferred_changed(
            void *data,
            struct ::wp_color_management_surface_feedback_v1 *object,
            uint32_t identity);
        struct ::wp_color_management_surface_feedback_v1 *m_wp_color_management_surface_feedback_v1;
    };

    class Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT wp_image_description_creator_icc_v1
    {
    public:
        wp_image_description_creator_icc_v1(struct ::wl_registry *registry, uint32_t id, int version);
        wp_image_description_creator_icc_v1(struct ::wp_image_description_creator_icc_v1 *object);
        wp_image_description_creator_icc_v1();

        virtual ~wp_image_description_creator_icc_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::wp_image_description_creator_icc_v1 *object);

        struct ::wp_image_description_creator_icc_v1 *object() { return m_wp_image_description_creator_icc_v1; }
        const struct ::wp_image_description_creator_icc_v1 *object() const { return m_wp_image_description_creator_icc_v1; }
        static wp_image_description_creator_icc_v1 *fromObject(struct ::wp_image_description_creator_icc_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_incomplete_set = 0, // incomplete parameter set
            error_already_set = 1, // property already set
            error_bad_fd = 2, // fd not seekable and readable
            error_bad_size = 3, // no or too much data
            error_out_of_file = 4, // offset + length exceeds file size
        };

        struct ::wp_image_description_v1 *create();
        void set_icc_file(int32_t icc_profile, uint32_t offset, uint32_t length);

    private:
        struct ::wp_image_description_creator_icc_v1 *m_wp_image_description_creator_icc_v1;
    };

    class Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT wp_image_description_creator_params_v1
    {
    public:
        wp_image_description_creator_params_v1(struct ::wl_registry *registry, uint32_t id, int version);
        wp_image_description_creator_params_v1(struct ::wp_image_description_creator_params_v1 *object);
        wp_image_description_creator_params_v1();

        virtual ~wp_image_description_creator_params_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::wp_image_description_creator_params_v1 *object);

        struct ::wp_image_description_creator_params_v1 *object() { return m_wp_image_description_creator_params_v1; }
        const struct ::wp_image_description_creator_params_v1 *object() const { return m_wp_image_description_creator_params_v1; }
        static wp_image_description_creator_params_v1 *fromObject(struct ::wp_image_description_creator_params_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_incomplete_set = 0, // incomplete parameter set
            error_already_set = 1, // property already set
            error_unsupported_feature = 2, // request not supported
            error_invalid_tf = 3, // invalid transfer characteristic
            error_invalid_primaries_named = 4, // invalid primaries named
            error_invalid_luminance = 5, // invalid luminance value or range
        };

        struct ::wp_image_description_v1 *create();
        void set_tf_named(uint32_t tf);
        void set_tf_power(uint32_t eexp);
        void set_primaries_named(uint32_t primaries);
        void set_primaries(int32_t r_x, int32_t r_y, int32_t g_x, int32_t g_y, int32_t b_x, int32_t b_y, int32_t w_x, int32_t w_y);
        void set_luminances(uint32_t min_lum, uint32_t max_lum, uint32_t reference_lum);
        void set_mastering_display_primaries(int32_t r_x, int32_t r_y, int32_t g_x, int32_t g_y, int32_t b_x, int32_t b_y, int32_t w_x, int32_t w_y);
        void set_mastering_luminance(uint32_t min_lum, uint32_t max_lum);
        void set_max_cll(uint32_t max_cll);
        void set_max_fall(uint32_t max_fall);

    private:
        struct ::wp_image_description_creator_params_v1 *m_wp_image_description_creator_params_v1;
    };

    class Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT wp_image_description_v1
    {
    public:
        wp_image_description_v1(struct ::wl_registry *registry, uint32_t id, int version);
        wp_image_description_v1(struct ::wp_image_description_v1 *object);
        wp_image_description_v1();

        virtual ~wp_image_description_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::wp_image_description_v1 *object);

        struct ::wp_image_description_v1 *object() { return m_wp_image_description_v1; }
        const struct ::wp_image_description_v1 *object() const { return m_wp_image_description_v1; }
        static wp_image_description_v1 *fromObject(struct ::wp_image_description_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_not_ready = 0, // attempted to use an object which is not ready
            error_no_information = 1, // get_information not allowed
        };

        enum cause {
            cause_low_version = 0, // interface version too low
            cause_unsupported = 1, // unsupported image description data
            cause_operating_system = 2, // error independent of the client
            cause_no_output = 3, // the relevant output no longer exists
        };

        void destroy();
        struct ::wp_image_description_info_v1 *get_information();

    protected:
        virtual void wp_image_description_v1_failed(uint32_t cause, const QString &msg);
        virtual void wp_image_description_v1_ready(uint32_t identity);

    private:
        void init_listener();
        static const struct wp_image_description_v1_listener m_wp_image_description_v1_listener;
        static void handle_failed(
            void *data,
            struct ::wp_image_description_v1 *object,
            uint32_t cause,
            const char *msg);
        static void handle_ready(
            void *data,
            struct ::wp_image_description_v1 *object,
            uint32_t identity);
        struct ::wp_image_description_v1 *m_wp_image_description_v1;
    };

    class Q_WAYLAND_CLIENT_COLOR_MANAGEMENT_V1_EXPORT wp_image_description_info_v1
    {
    public:
        wp_image_description_info_v1(struct ::wl_registry *registry, uint32_t id, int version);
        wp_image_description_info_v1(struct ::wp_image_description_info_v1 *object);
        wp_image_description_info_v1();

        virtual ~wp_image_description_info_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::wp_image_description_info_v1 *object);

        struct ::wp_image_description_info_v1 *object() { return m_wp_image_description_info_v1; }
        const struct ::wp_image_description_info_v1 *object() const { return m_wp_image_description_info_v1; }
        static wp_image_description_info_v1 *fromObject(struct ::wp_image_description_info_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

    protected:
        virtual void wp_image_description_info_v1_done();
        virtual void wp_image_description_info_v1_icc_file(int32_t icc, uint32_t icc_size);
        virtual void wp_image_description_info_v1_primaries(int32_t r_x, int32_t r_y, int32_t g_x, int32_t g_y, int32_t b_x, int32_t b_y, int32_t w_x, int32_t w_y);
        virtual void wp_image_description_info_v1_primaries_named(uint32_t primaries);
        virtual void wp_image_description_info_v1_tf_power(uint32_t eexp);
        virtual void wp_image_description_info_v1_tf_named(uint32_t tf);
        virtual void wp_image_description_info_v1_luminances(uint32_t min_lum, uint32_t max_lum, uint32_t reference_lum);
        virtual void wp_image_description_info_v1_target_primaries(int32_t r_x, int32_t r_y, int32_t g_x, int32_t g_y, int32_t b_x, int32_t b_y, int32_t w_x, int32_t w_y);
        virtual void wp_image_description_info_v1_target_luminance(uint32_t min_lum, uint32_t max_lum);
        virtual void wp_image_description_info_v1_target_max_cll(uint32_t max_cll);
        virtual void wp_image_description_info_v1_target_max_fall(uint32_t max_fall);

    private:
        void init_listener();
        static const struct wp_image_description_info_v1_listener m_wp_image_description_info_v1_listener;
        static void handle_done(
            void *data,
            struct ::wp_image_description_info_v1 *object);
        static void handle_icc_file(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            int32_t icc,
            uint32_t icc_size);
        static void handle_primaries(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            int32_t r_x,
            int32_t r_y,
            int32_t g_x,
            int32_t g_y,
            int32_t b_x,
            int32_t b_y,
            int32_t w_x,
            int32_t w_y);
        static void handle_primaries_named(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            uint32_t primaries);
        static void handle_tf_power(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            uint32_t eexp);
        static void handle_tf_named(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            uint32_t tf);
        static void handle_luminances(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            uint32_t min_lum,
            uint32_t max_lum,
            uint32_t reference_lum);
        static void handle_target_primaries(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            int32_t r_x,
            int32_t r_y,
            int32_t g_x,
            int32_t g_y,
            int32_t b_x,
            int32_t b_y,
            int32_t w_x,
            int32_t w_y);
        static void handle_target_luminance(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            uint32_t min_lum,
            uint32_t max_lum);
        static void handle_target_max_cll(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            uint32_t max_cll);
        static void handle_target_max_fall(
            void *data,
            struct ::wp_image_description_info_v1 *object,
            uint32_t max_fall);
        struct ::wp_image_description_info_v1 *m_wp_image_description_info_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
